@@ -85,7 +85,8 @@ def add_observations(sfrdata, data, flowline_routing=None,
     reach_data = sfrdata.reach_data.copy()
 
     # allow input via a list of tables or single table
-    data = read_tables(data)
+    data = read_tables(data, dtype={obsname_column: object})
+    assert data[obsname_column].dtype == np.object
 
     # read reach geometries from a shapefile
     if sfrlines_shapefile is not None:
@@ -263,7 +264,7 @@ def locate_sites(site_data,
         writing output_csv_file and output_shape_file.
     perimeter_buffer : scalar
         Exclude flows within this distance of perimeter defined
-        by active_area_shapefile. For example, a value of 1000 would
+        by active_area. For example, a value of 1000 would
         mean that sites must be at least 1 km inside of the active area perimeter to
         be included.
     distance_threshold : scalar
@@ -300,7 +301,7 @@ def locate_sites(site_data,
     else:
         raise TypeError('Datatype for site_data not understood: {}'.format(site_data))
 
-    # reproject if crs are available
+    # to_crs if crs are available
     if locsproj4 is not None and sfrproj4 is not None:
         locs['geometry'] = project(locs.geometry.values, locsproj4, sfrproj4)
 
@@ -410,6 +411,7 @@ def write_gage_package(location_data,
     with open(gage_namfile_entries_file, 'w') as output:
         for i, f in enumerate(df.gagefile.values):
             output.write('DATA  {:d} {}\n'.format(gage_data.unit[i], f))
+    return gag
 
 
 def write_mf6_sfr_obsfile(observation_locations,
