@@ -268,7 +268,7 @@ class SFRData(DataPackage):
 
     @property
     def modflow_sfr2(self):
-        """Flopy modflow_sfr2 instance."""
+        """A `flopy.modflow.mfsfr2.ModflowSfr2` represenation of the sfr dataset."""
         if self._ModflowSfr2 is None:
             self.create_modflow_sfr2()
         return self._ModflowSfr2
@@ -905,7 +905,7 @@ class SFRData(DataPackage):
             features = self.grid.df.loc[self.reach_data.node, 'geometry'].tolist()
             txt = method
 
-        # reproject features if they're not in the same crs
+        # to_crs features if they're not in the same crs
         if raster_crs != self.crs:
             features = project(features,
                                self.crs.proj_str,
@@ -1201,7 +1201,9 @@ class SFRData(DataPackage):
             model = None
 
         # create an SFRData instance
-        to_sfr_kwargs = get_input_arguments(cfg, sfrmaker.Lines.to_sfr)
+        to_sfr_kwargs = cfg.copy()
+        to_sfr_kwargs.update(cfg.get('options', {}))
+        to_sfr_kwargs = get_input_arguments(to_sfr_kwargs, sfrmaker.Lines.to_sfr)
         to_sfr_kwargs['active_area'] = cfg.get('active_area', {}).get('filename')
         to_sfr_kwargs['model'] = model
         to_sfr_kwargs['grid'] = grid
@@ -1212,7 +1214,7 @@ class SFRData(DataPackage):
         sfrdata._shapefiles_path = os.path.join(output_path, sfrdata._shapefiles_path)
 
         # setup elevations
-        if cfg.get('set_streambed_top_elevations_from_dem', False):
+        if cfg.get('options', cfg).get('set_streambed_top_elevations_from_dem', False):
             error_msg = ("set_streambed_top_elevations_from_dem=True "
                          "requires a dem: block.")
             if 'dem' not in cfg:
@@ -1445,13 +1447,52 @@ class SFRData(DataPackage):
                       write_observations_input=True,
                       external_files_path=None, gage_starting_unit_number=None,
                       **kwargs):
-        """Write SFR package input.
+        """Write an SFR package input file.
 
         Parameters
         ----------
-        version : str
-            'mf2005' or 'mf6'
-        """
+        filename : str, optional
+            File name for SFR package. By default None, in which
+            case the filename of the attached :py:class:`flopy.modflow.mfsfr2.ModflowSfr2` instance
+            (``SFRData.modflow_sfr2.fn_path``) is used.
+        version : str, optional, {'mf2005', 'mfnwt', 'mf6'}
+            MODFLOW version for the SFR package, by default 'mf2005'
+        idomain : ndarray, optional
+            3D numpy array designating active cells (idomain==1).
+            SFR reaches in inactive cells will be written with 'none' in the cellid field.
+            by default None
+        options : list, optional
+            List of strings to write to the MODFLOW-6 SFR options block. For example::
+
+                options=['save_flows',
+                         'BUDGET FILEOUT model.sfr.cbc',
+                         'STAGE FILEOUT model.sfr.stage.bin']
+
+            * An appropriate unit_conversion is written by default.
+            * Cell-by-cell budget and stage output files are also added by default if not specified.
+            * If the :attr:`SFRData.observations` is populated and `write_observations_input=True`,
+              the observation input file is also added.
+            * All files added by default are placed in the same path as the SFR package file. If another
+              location is desired, the file(s) need to be specified explicitly as strings in the options
+              list.
+
+            See MODFLOW-6 documentation for other options.
+            By default None.
+        run_diagnostics : bool, optional
+            Option to run the :ref:`diagnostic checks <Running diagnostics>`.
+            by default True
+        write_observations_input : bool, optional
+            Option to write input to the the MODFLOW-2005 gage package or MODFLOW-6 Observations Process.
+            Requires attached observations, as added through the :py:meth:`SFRData.add_observations` method.
+            by default True
+        external_files_path : str, optional
+            Path for writing an external file for packagedata, relative to the location of the SFR package file.
+            If specified, an open/close statement referencing the file is written to the packagedata block.
+            By default, None (packagedata table is written to the SFR package file)
+        gage_starting_unit_number : int, optional
+            Starting unit number for gage output files, 
+            by default None
+        """        
         print('SFRmaker v. {}'.format(sfrmaker.__version__))
         # run the flopy SFR diagnostics
         if run_diagnostics:
@@ -1467,6 +1508,7 @@ class SFRData(DataPackage):
 
         if filename is None:
             filename = self.modflow_sfr2.fn_path
+
         if gage_starting_unit_number is None:
             gage_starting_unit_number = self.gage_starting_unit_number
         if version in {'mf2005', 'mfnwt'}:
@@ -1485,14 +1527,25 @@ class SFRData(DataPackage):
             from .mf5to6 import Mf6SFR
             if options is None:
                 # save budget and stage output by default
+                # if options weren't specified, assume the output files
+                # should be in the same location as the SFR package file
+                # (do the same for obs below)
+                _, just_the_filename = os.path.split(filename)
                 options = ['save_flows',
-                           'BUDGET FILEOUT {}.cbc'.format(filename),
-                           'STAGE FILEOUT {}.stage.bin'.format(filename),
+                           'BUDGET FILEOUT {}.cbc'.format(just_the_filename),
+                           'STAGE FILEOUT {}.stage.bin'.format(just_the_filename),
                            ]
             if write_observations_input and len(self.observations) > 0:
-                obs_input_filename = filename + '.obs'
+                if 'obs6 filein' not in ''.join(options).lower():
+                    obs_input_filename = filename + '.obs'
+                    options.append('OBS6 FILEIN {}'.format(just_the_filename + '.obs'))
+                else:
+                    for entry in options:
+                        if 'obs6 filein' in entry.lower():
+                            break
+                    _, _, obs_input_filename = entry.split()
                 self.write_mf6_sfr_obsfile(filename=obs_input_filename)
-                options.append('OBS6 FILEIN {}'.format(obs_input_filename))
+
 
             sfr6 = Mf6SFR(SFRData=self, period_data=self.period_data,
                           idomain=idomain,
@@ -1502,6 +1555,15 @@ class SFRData(DataPackage):
             sfr6.write_file(filename=filename, external_files_path=external_files_path)
 
     def write_tables(self, basename=None):
+        """Write :py:attr:`~SFRData.reach_data`, :py:attr:`~SFRData.segment_data`,
+        and :py:attr:`~SFRData.period_data` (if populated) to csv files. 
+
+        Parameters
+        ----------
+        basename : str, optional
+            Base name for csv files, by default None, in which case
+            :py:attr:`SFRData.package_name` is used.
+        """
         if basename is None:
             output_path = self._tables_path
             if not os.path.isdir(output_path):
@@ -1524,6 +1586,23 @@ class SFRData(DataPackage):
 
     def write_gage_package(self, filename=None, gage_package_unit=25,
                            gage_starting_unit_number=None):
+        """Write observation input for the MODFLOW-2005 Gage Package.
+
+        Parameters
+        ----------
+        filename : str, optional
+            Gage package file, by default None, in which case
+            :py:attr:`SFRData.observations_file` is used.
+        gage_package_unit : int, optional
+            Unit number for Gage Package, by default 25
+        gage_starting_unit_number : int, optional
+            Starting unit number for gage output files, 
+            by default None
+
+        Returns
+        -------
+        gag : :py:class:`flopy.modflow.mfgage.ModflowGage` instance
+        """        
         if filename is None:
             filename = self.observations_file
         else:
