@@ -60,9 +60,9 @@ def test_from_yaml(shellmound_config):
     assert os.path.exists('sfrmaker/test/temp/shellmound/tables/shellmound_sfr_reach_data.csv')
     options = read_mf6_block(sfr_package_file, 'options')
     assert 'save_flows' in options
-    assert options['budget'] == ['fileout', '../../temp/shellmound/shellmound.sfr.cbc']
-    assert options['stage'] == ['fileout', '../../temp/shellmound/shellmound.sfr.stage.bin']
-    assert options['obs6'] == ['filein', '../../temp/shellmound/shellmound.sfr.obs']
+    assert options['budget'] == ['fileout', 'shellmound.sfr.cbc']
+    assert options['stage'] == ['fileout', 'shellmound.sfr.stage.bin']
+    assert options['obs6'] == ['filein', 'shellmound.sfr.obs']
 
 
 def get_package_version(sfr_package_file):
@@ -117,6 +117,14 @@ def test_from_config(shellmound_config, package_version, model_cfg):
         os.remove('../../temp/shellmound/shellmound.gage')
         os.remove('../../temp/shellmound/shellmound.gage.namefile_entries')
 
+    # check that add_outlets works
+    add_outlets = cfg['options']['add_outlets']
+    if not isinstance(add_outlets, list):
+        add_outlets = [add_outlets]
+    for outlet_id in add_outlets:
+        assert sfrdata.reach_data.loc[sfrdata.reach_data.line_id == outlet_id,
+                                      'outseg'].sum() == 0
+
 
 @pytest.mark.parametrize('config_file,dem', (
         ('examples/tylerforks/tf_sfrmaker_config.yml', True),
